#!/usr/bin/env bash

TEMP=$(getopt -o "dsph" \
              --long help,disk-mode,stacks,pages,iterations:,value-size: \
              -n "$(basename "$0")" -- "$@")

if [ $? != 0 ] ; then echo "Terminating..." >&2 ; exit 1 ; fi

eval set -- "$TEMP"

ITERATIONS=10000
VALUE_SIZE="2K"
DISK_MODE=""
VALGRIND_MEASURE=""

while true; do
  case "$1" in
    -d | --disk-mode ) DISK_MODE="--disk"; shift ;;
    -s | --stacks ) VALGRIND_MEASURE="--stacks=yes"; shift ;;
    -p | --pages ) VALGRIND_MEASURE="--pages-as-heap=yes"; shift ;;
    --value-size ) VALUE_SIZE="$2"; shift 2 ;;
    --iterations ) ITERATIONS="$2"; shift 2 ;;
    -h | --help )
      prog=$(basename -s.wrapper "$0")
      echo "Usage: $prog [options...]"
      echo "     --iterations <count> Number of key-value pairs to insert"
      echo "     --value-size <size> Size of the values for the keys, e.g. 2K"
      echo "         Can also be set by using AIRGAPPED_NAME environment variable"
      echo " -d, --disk-mode Should dqlite run on disk mode(Experimental!)"
      echo " -s, --stacks Valgrind will measure heap and the stack with this flag."
      echo " -p, --pages Valgrind will treat pages as a heap, and will measure memory usage down to mmalloc calls."
      echo
      exit ;;
    -- ) shift; break ;;
    * ) break ;;
  esac
done

trap "trap - SIGTERM && kill -- -$$" SIGINT SIGTERM EXIT

sudo rm -rf /tmp/dqlite-demo

mkdir -p ./dqlite-demo$DISK_MODE-$ITERATIONS

units="leader voter-1 voter-2"

for unit in $units;
do
  echo "Iterations,USS,PSS,RSS" > ./dqlite-demo$DISK_MODE-$ITERATIONS/smem-dqlite-demo-$unit.csv
done

sudo valgrind --tool=massif $VALGRIND_MEASURE --massif-out-file=./dqlite-demo$DISK_MODE-$ITERATIONS/dqlite-demo-leader dqlite-demo $DISK_MODE --api 127.0.0.1:8001 --db 127.0.0.1:9001 > /dev/null 2>&1 &
sudo valgrind --tool=massif $VALGRIND_MEASURE --massif-out-file=./dqlite-demo$DISK_MODE-$ITERATIONS/dqlite-demo-voter-1 dqlite-demo $DISK_MODE --api 127.0.0.1:8002 --db 127.0.0.1:9002 --join 127.0.0.1:9001 > /dev/null 2>&1 &
sudo valgrind --tool=massif $VALGRIND_MEASURE --massif-out-file=./dqlite-demo$DISK_MODE-$ITERATIONS/dqlite-demo-voter-2 dqlite-demo $DISK_MODE --api 127.0.0.1:8003 --db 127.0.0.1:9003 --join 127.0.0.1:9001 > /dev/null 2>&1 &

sleep 5

perl -077 -ne 'print unpack("b*")' < /dev/urandom | head -c$VALUE_SIZE > /tmp/random-data

for i in $(seq 1 $ITERATIONS); do
  curl -X PUT --data-binary "@/tmp/random-data" http://127.0.0.1:8001/key-$i &> /dev/null;

  if [[ $(( i % 500 )) == 0 ]];
  then
    for unit in $units;
    do
      smo=$(sudo smem -a | grep valgrind.bin | grep dqlite-demo-$unit | tr -s ' ')
      mem_dqlite=($smo)
      echo "$i" "${mem_dqlite[@]: -3}" | sed -e 's/\s\+/,/g' >> ./dqlite-demo$DISK_MODE-$ITERATIONS/smem-dqlite-demo-$unit.csv
    done
  fi
  
done